/**
 * @file Container.cpp
 * @author apio (cloudapio.eu)
 * @brief A container widget to pad and align objects inside it.
 *
 * @copyright Copyright (c) 2023, the Luna authors.
 *
 */

#include <ui/Container.h>

namespace ui
{
    Container::Container(Window* window, Widget* parent) : ui::Widget(window, parent)
    {
    }

    void Container::recalculate_true_rects()
    {
        if (!m_widget) return;
        m_widget->rect() = ui::align(m_rect, m_widget->pseudo_rect(), m_valign, m_halign);
        m_widget->recalculate_true_rects();
    }

    void Container::recalculate_pseudo_rects()
    {
        if (m_widget) m_widget->recalculate_pseudo_rects();

        m_pseudo_rect = preferred_rect();
    }

    Result<EventResult> Container::handle_mouse_move(Point position)
    {
        if (m_widget->rect().contains(position)) return m_widget->handle_mouse_move(position);
        return ui::EventResult::DidNotHandle;
    }

    Result<EventResult> Container::handle_mouse_leave()
    {
        return m_widget->handle_mouse_leave();
    }

    Result<EventResult> Container::handle_mouse_down(Point position, int buttons)
    {
        if (m_widget->rect().contains(position)) return m_widget->handle_mouse_down(position, buttons);
        return ui::EventResult::DidNotHandle;
    }

    Result<EventResult> Container::handle_mouse_up(Point position, int buttons)
    {
        if (m_widget->rect().contains(position)) return m_widget->handle_mouse_up(position, buttons);
        return ui::EventResult::DidNotHandle;
    }

    Result<EventResult> Container::handle_key_event(const ui::KeyEventRequest& request)
    {
        return m_widget->handle_key_event(request);
    }

    Result<void> Container::draw(Canvas& canvas)
    {
        auto rect = ui::Rect { m_widget->rect().pos.x - m_rect.pos.x, m_widget->rect().pos.y - m_rect.pos.y,
                               m_widget->rect().width, m_widget->rect().height };
        auto subcanvas = canvas.subcanvas(rect);
        return m_widget->draw(subcanvas);
    }

}
